<?php

class DFDebug {
	// If is_admin(), this will contain object for admin-related functionality
	var $admin = null;

	var $settings = array();
	var $enabled = true;

	var $default_settings = array(
		'show_admin_footer' => array(),
		'savequeries'       => false,
		'all_action'        => false
	);

	function __construct() {
		$this->settings = $this->get_settings();

		// Backend functionality
		if ( is_admin() ) {
			require( DFDEBUG_DIR . '/admin.php' );
			$this->admin = new DFDebug_Admin( $this );
		}

		$this->setup_menu_bar();

		// Any front-end hooks should require explicit activation.
//		add_action( 'all', array(&$this, 'all_action' ) );
//		add_action( 'wp_footer', array( &$this, 'show_queries' ), 999999999 );
//		if ( !defined('SAVEQUERIES') ) define( 'SAVEQUERIES', true );

	}

	function get_settings() {
		$settings = get_option( 'dfdebug_settings', array() );
		$user_settings = $this->get_user_settings();

		return wp_parse_args( $user_settings, $settings, $this->default_settings );
	}

	function get_user_settings() {
		$uid = get_current_user_id();
		$settings = $uid ? (array) get_user_meta( $uid, 'dfdebug_settings', true ) : array();

		return $settings;
	}

	function setup_menu_bar() {
		// Core hook
		add_action( 'admin_bar_menu', array( &$this, 'admin_bar_menu' ), apply_filters( 'dfdebug_admin_bar_priority', 999999 ) );

		// Custom hook
		add_filter( 'dfdebug_menu_items', array( &$this, 'admin_menu_items' ), 10, 2 );
	}

	function admin_bar_menu( &$menu_bar ) {
		// Setup parent menu item
		$parent_id = 'dfdebug_main';
		$menu_bar->add_node( array(
			'id' => $parent_id,
			'title' => 'Debug'
		) );

		$menu_items = apply_filters( 'dfdebug_menu_items', array(), $parent_id );

		foreach( $menu_items as $item )
			$menu_bar->add_node( $item );

	}

	function admin_menu_items( $items, $parent_id ) {
		global $wpdb;

		$is_win = 'WIN' === strtoupper( substr( PHP_OS, 0, 3 ) );
		if ( ! $is_win ) {
			$items += array(
				array(
					'id'     => 'dfdebug_system_load',
					'title'  => 'Load: ' . implode( ' ', $this->system_load() ),
					'parent' => $parent_id
				)
			);
		}

		$items += array(
			array(
				'id'     => 'dfdebug_peak_memory',
				'title'  => 'Peak memory: ' . $this->peak_memory_mb() . 'M',
				'parent' => $parent_id
			),
			array(
				'id'     => 'dfdebug_process_time',
				'title'  => 'Processing time: ' . timer_stop() . ' sec',
				'parent' => $parent_id
			),
			array(
				'id'     => 'dfdebug_query_count',
				'title'  => 'Queries: ' . $wpdb->num_queries,
				'parent' => $parent_id
			)
		);

		if ( current_user_can( 'manage_options' ) ) {
			$items[] = array(
				'id'     => 'dfdebug_admin',
				'title'  => 'Settings',
				'href'   => admin_url( 'admin.php?page=dfdebug-admin' ),
				'parent' => $parent_id
			);
		}

		return $items;
	}

	function safer_memory_get_peak_usage() {
		static $mem = 0;

		if ( function_exists( 'memory_get_peak_usage' ) )
			return memory_get_peak_usage();

		$new_mem = memory_get_usage();
		if ( $new_mem > $mem )
			$mem = $new_mem;

		return $mem;
	}

	function peak_memory_mb() {
		return round( (float) $this->safer_memory_get_peak_usage() / pow( 1024, 2 ), 3 );
	}

	function system_load() {
		$output = shell_exec( 'cat /proc/loadavg' );
		return array_slice( explode( ' ', $output ), 0, 3 );
	}

	// For use in 'all' actions or other situations where a profiling function is hooked to multiple actions.
	function current_filter() {
		return $GLOBALS['wp_current_filter'][0];
	}

	// Based on wp_debug_mode()
	function set_debug_mode( $enable, $display = false, $log_file = false ) {
		if ( $enable ) {
			// E_DEPRECATED is a core PHP constant in PHP 5.3. Don't define this yourself.
			// The two statements are equivalent, just one is for 5.3+ and for less than 5.3.
			if ( defined( 'E_DEPRECATED' ) )
				error_reporting( E_ALL & ~E_DEPRECATED & ~E_STRICT );
			else
				error_reporting( E_ALL );

			if ( $display )
				ini_set( 'display_errors', 1 );
			elseif ( null !== WP_DEBUG_DISPLAY )
				ini_set( 'display_errors', 0 );

			if ( $log_file ) {
				$file = is_bool( $log_file ) ? WP_CONTENT_DIR . '/debug.log' : $log_file;
				ini_set( 'log_errors', 1 );
				ini_set( 'error_log', $file );
			}
		} else {
			error_reporting( E_CORE_ERROR | E_CORE_WARNING | E_COMPILE_ERROR | E_ERROR | E_WARNING | E_PARSE | E_USER_ERROR | E_USER_WARNING | E_RECOVERABLE_ERROR );
		}
	}
}
